/*===========================================================
  c     First attempt at 2D analogue of electro-static
  c     trap described by Dan Heinzen, Dec 11 1998.
  c
  c     Solves 2D Poisson equation with internal Dirichlet
  c     "boundaries" representing infinitely thin conducting
  c     plates.
  c
  c     Copyright 1998-  Matthew W Choptuik, UT Austin
  C               1999- Scott A. Klasky, Syracuse University
  c=========================================================== */
#include <stdio.h>
#include <math.h>
#include  "sv2_api.h"
static void setup1(int nx,int ny, int rank,double *hx,double *hy);
static void relax0(int nx,int ny,double hx,double hy,double omega,double *resnrm);

#define LTRACE TRUE
#define maxrank 2
int rank;
int shp[maxrank];
double bbox[2*maxrank],bb[maxrank];
#define  maxn 2049
double phi[maxn][maxn], chars[maxn][maxn],x[maxn],y[maxn];
double phi1d[maxn*maxn],phis[maxn];

void main (int argc, char *argv[]) {

  /* int mymod; */
  int level, ofreq=0, ofreq1,ofreq2;
  double omega, epsi;
  int n,nx,ny;
  double hx,hy,resnrm;
  int iter;
  int ii,jj,ll,lll;
  double time;

  int jin,jin2,jin3,_test,frameNumber=0;

  /*
  level = 5;
  omega = 1.7;
  epsi = 0.9;
  */
  level = 5;
  omega = 1.7;
  epsi = 0.9;
  ofreq1 = 20;
  ofreq2 = 400;
printf("------------------changed by jin---------------------------\n");
/*
  if (argc < 2) {
    printf("'usage: etrap2d <level> [<omega> <epsi> <ofreq>]\n");
    printf("level=3 by default\n");
  }
  sscanf(argv[1],"%d",&level);
  if (level <3) {
    printf("'usage: etrap2d <level> [<omega> <epsi> <ofreq>]\n");
    printf("level=3 by default\n");
  }
*/
  n = (int) (pow(2,level) + 1);
  if (n > maxn) {
    printf( "etrap2d: n = %d  exceeds %d \n",n, maxn);
  }
/*
  if (argc >1) {
    sscanf(argv[2],"%lf",&omega);
  }
  if (argc>2) {
    sscanf(argv[3],"%lf",&epsi);
  }
  if (argc>3) {
    sscanf(argv[4],"%d",&ofreq);
  }
*/

#ifdef LTRACE
  printf("etrap: level = %d\n",level);
  printf("etrap: n = %d\n",n);
  printf("etrap: omega = %lf\n",omega);
  printf("etrap: epsi = %lf\n",epsi);
  printf("etrap: ofreq = %lf\n",ofreq);
#endif

  nx = n;
  ny = n;
  setup1(nx,ny,rank,&hx,&hy);
  bb[0] = bbox[0];
  bb[1] = bbox[1];
  ofreq = ofreq1;
  iter   = 0;
  resnrm = 1.0 + epsi;
  while ( resnrm > epsi ) {
    relax0(nx,ny,hx,hy,omega,&resnrm);
    iter ++;

    if (iter == 100) ofreq = ofreq2;
    time = iter;
    if (iter % ofreq == 0) {
      printf ("etrap2d: iter= %d  |res| = %lg\n",iter,resnrm);
  ll = 0;lll=0;
  for (ii=0;ii<nx;ii++) {
    for (jj=0;jj<ny;jj++) {
       phi1d[ll] = phi[jj][ii];
       if (jj==(ny-1)/2) {
         phis[lll] = phi[jj][ii];
         lll++;
       }
       ll++;
    }
  }
/*added by jin*/
for(jin=0;jin<10000;jin++){
    for(jin2=0;jin2<5000;jin2++)
         _test = 0;
}
    frameNumber++;


  sv2_bbser2d("phi", (double)iter, bbox, ny, nx, phi1d);
  sv2_bbser("phi1d", (double)iter, bb, nx, phis);
    }
  }
  ll = 0;
  for (ii=0;ii<nx;ii++) {
    for (jj=0;jj<ny;jj++) {
       phi1d[ll] = phi[jj][ii];
       ll++;
    }
  }
  frameNumber++;
  sv2_bbser2d("phi", (double)iter, bbox, ny, nx, phi1d);
  printf ("etrap2d: iter= %d  |res| = %lg\n",iter,resnrm);
  // gft_out('phi',time,shp,rank,phi);
  printf("Total frame number %d\n",frameNumber);

}

/*
  c===========================================================
  c     Relaxation routine.  Uses characteristic function
  c     to enumerate dictinct "grid point types":
  c
  c        chars =  0.0d0   :  Interior
  c        chars =  1.0d0   :  Dirichlet
  c
  c     Assumes that Dirichlet outer boundary values have
  c     been previously set.
  c===========================================================
*/
static void relax0(int nx,int ny,double hx,double hy,double omega,double *resnrm) {
  int nnrm;
  double phigs, hxm2,hym2,res,jacel;
  int i,j;

  hxm2 = 1.0/hx/hx;
  hym2 = 1.0/hy/hy;
  jacel = -2.0*(hxm2*hym2);
  *resnrm = 0.0;
  nnrm = 0;
  for (j=1;j<ny-1;j++) {
    for (i=1;i<nx-1;i++) {
      if ( chars[j][i] == 0.0) {
	res = hxm2 * (phi[j][i+1] - 2.0*phi[j][i] + phi[j][i-1]) + hym2 *
	  (phi[j+1][i] - 2.0*phi[j][i] + phi[j-1][i] );
	*resnrm = *resnrm + res * res;
	nnrm = nnrm + 1;
	phigs = phi[j][i] - res/jacel;
	phi[j][i] = omega * phigs + (1.0-omega)*phi[j][i];
      }else if (chars[j][i] == 1.0) {
      } else {
	printf("relax0: unexpected chars value %d\n",chars[j][i]);
      }
    }
  }
}
/*
  c===========================================================
  c     Setup routine.  Initializes potential and charac-
  c     terstic function.
  c===========================================================
*/
static void setup1(int nx,int ny, int rank,double *hx,double *hy){

  int i,j;
  int i1,i2,jbgn,jend;
  int j1,j2,ibgn,iend;

  rank = 2;

  shp[0]  = nx;
  shp[1]  = ny;

  bbox[0] = -1.0;
  bbox[1] =  1.0;
  bbox[2] = -1.0;
  bbox[3] =  1.0;
  *hx = (bbox[1]-bbox[0])/(nx -1);
  *hy = (bbox[3]-bbox[2])/(nx -1);
  for ( i = 0;i<nx;i++ ) {
    x[i] = bbox[0] + *hx * i;
  }
  for ( i = 0;i<ny;i++ ) {
    y[i] = bbox[2] + *hy * i;
  }

  for (i=0;i<nx;i++) {
    for (j=0;j<ny;j++) {
      phi[j][i] = (double)(j%2) + (double)(i%2);
      if ( (i==0) || (j==0) || (i==nx-1) || (j==ny-1) ) phi[j][i] = 0.0;
      chars[j][i] = 0.0;
    }
  }
  i1   = 1 + 2 * (nx - 1) / 8 - 1;
  i2   = nx - 2 * (nx - 1) / 8 - 1;
  jbgn = (ny - 1) / 4 + 1 - 1;
  jend = ny - (ny - 1) / 4 - 1;

  j1   = 1 + (ny - 1) / 4 - 1;
  j2   = ny - (ny - 1) / 4-1;
  ibgn = 3 * (nx - 1) / 8 + 1 - 1;
  iend = nx - 3 * (nx - 1) / 8 - 1;

  for (j=jbgn;j<=jend;j++) {
    phi[j][i1]  = 2.0;
    chars[j][i1] = 1.0;
    phi[j][i2]  = 2.0;
    chars[j][i2] = 1.0;
  }
  for (i=ibgn;i<=iend;i++) {
    phi[j1][i]  = -3.0;
    chars[j1][i] =  1.0;
    phi[j2][i]  = -3.0;
    chars[j2][i] =  1.0;
  }
}

